import json
from json.decoder import JSONDecodeError
from urllib.parse import urlencode

import requests
from bs4 import BeautifulSoup


class HuaDao:
    """
    华道返回 xml
    """

    @staticmethod
    def get_token(AppID, AppSecret, Key):
        access_token = 'http://opensdk.emay.cn:9080/HD_GetAccess_Token.asmx/GetACCESS_TOKEN'
        full_url = '?'.join((access_token, urlencode({
            'AppID': AppID,
            'AppSecret': AppSecret,
            'Key': Key
        })))
        r = requests.get(full_url)
        return r.text

    @staticmethod
    def gen_full_url(url, biz_data):
        """
        you can get full_url by this function return
        :param url: base url
        :param biz_data:
        :return:
        """
        return '?'.join((url, urlencode(biz_data)))

    @classmethod
    def _request(cls, url, biz_data):
        full_url = cls.gen_full_url(url, biz_data)
        r = requests.get(full_url)
        if r.ok:
            try:
                bs = BeautifulSoup(r.text, features='html.parser')
                return json.loads(bs.text)
            except JSONDecodeError as e:
                raise TypeError(f'JSONDecodeError, raw resp {r.text}')
            except Exception as e:
                raise SystemError(f'raw resp1 {r.text}')
        else:
            raise ValueError(f'raw resp2 {r.text}')


class BankFiveElements(HuaDao):
    url = 'http://opensdk.emay.cn:9099/SF_YZ_API/SFService.asmx/Get_EMW_GetBank_Card_Five_RZ'

    def __init__(self, name, idcard, cardNo, phone, bankAccountType, token):
        self.token = token
        self.biz_data = {
            'name': name,
            'idcard': idcard,
            'cardNo': cardNo,
            'phone': phone,
            'bankAccountType': bankAccountType,
            'ACCESS_TOKEN': self.token
        }

    def request(self):
        return self._request(self.url, self.biz_data)


if __name__ == '__main__':
    a1 = BankFiveElements('a', 'b', 'c', 'd', 'f', '')
    a1.request()



try:
    from service.ocr.cloud_walk.idcard import IdCard
    from service.ocr.cloud_walk.biz_license import BizLicense
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.ocr.cloud_walk.idcard import IdCard
    from pysubway.service.ocr.cloud_walk.biz_license import BizLicense



import json


class Dict(dict):

    def sort_keys(self, sort_by_ascii=True) -> str:
        """
        sort dict value by dict key's order in ascii.
        :param data_dict:
        :return:
        """
        if sort_by_ascii:
            sign_data = ''
            for key in sorted(self.keys()):
                value = self[key]
                if isinstance(value, (dict, list)):
                    sign_data += json.dumps(self[key], ensure_ascii=False, sort_keys=True).encode('utf-8')
                else:
                    sign_data += str(value)
            return sign_data
        else:
            raise NotImplementedError(f'sort_by_ascii {sort_by_ascii}')



try:
    from service import bangsheng
    from service.ocr import cloud_walk
    from service import tonglian
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service import bangsheng
    from pysubway.service.ocr import cloud_walk
    from pysubway.service import tonglian


import os
from typing import Dict, Any

import chardet
from chardet.gb2312prober import GB2312Prober
from chardet.latin1prober import Latin1Prober


class Open:
    """
    example:
    >>>with Open(__file__) as f:
    >>> for i in range(10):
    >>>     print(f.readline())
    """

    def __init__(self, file):
        self.file = file
        self._encoding_type = None
        self.encoding = 'utf-8'
        self.file_encoding_type = self.encoding_type['encoding']
        self.gdk_compatibility = [
            # ascii
            Latin1Prober().charset_name,
            # gbk
            GB2312Prober().charset_name,
            'ISO-8859-9',
        ]
        if self.file_encoding_type in self.gdk_compatibility:
            self.encoding = 'gbk'
        try:
            self.f = open(self.file, encoding=self.encoding)
        except UnicodeEncodeError as e:
            print(e)
            self.f = open(self.file, encoding=self.file_encoding_type)

    @property
    def encoding_type(self) -> Dict[str, Any]:
        """
        {'encoding': 'ascii', 'confidence': 1.0, 'language': ''}
        :param self:
        :return:
        """
        if not self._encoding_type:
            with open(self.file, mode='rb') as f:
                # must read all, or the detect result is imprecision
                self._encoding_type = chardet.detect(f.read())
        return self._encoding_type

    def __enter__(self):
        return self.f

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.f.close()


def file_dirname(file):
    return os.path.split(file)[0]


def join_path(dirname, filename):
    return os.path.join(dirname, filename)



import os
import secrets

import pandas as pd

from .file import Open

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


def csv2dataframe(file: str) -> pd.DataFrame:
    tmp_csv = os.path.join(THIS_DIR, '.'.join((secrets.token_hex(), 'csv')))
    with Open(file) as f:
        with open(tmp_csv, mode='w', encoding='utf-8') as fc:
            fc.write(f.read())
        tmp = pd.read_csv(tmp_csv).iterrows()
        frame = []
        for i in tmp:
            series = {}
            for w in i[1].iteritems():
                k = "".join(str(w[0]).split())
                v = "".join(str(w[1]).split())
                series[k] = v.strip('"')
            frame.append(series)
        return pd.DataFrame(frame)


def csv2xlsx(old: str, new: str, clean: bool = False) -> None:
    if not clean:
        if not isinstance(old, str) and not old.endswith('csv'):
            raise Exception(f'old {old} file must be csv')
        if not isinstance(new, str) and not new.endswith('xlsx'):
            raise Exception(f'old {new} file must be xlsx')
        pd.read_csv(old).to_excel('./tmp.xlsx')
    else:
        csv2dataframe(old).to_excel(new)



import re
import secrets
import uuid
from typing import Union

PATTERN_CHINESE_NAME = re.compile(r'[^\u4e00-\u9fa5·]+')


def remove_invisible(s: str):
    """
    support remove \t \n \xa0 ...
    :param s:
    :return:
    """
    return ''.join(s.split())


def unique_id():
    return uuid.uuid1()


def to_bytes(s: Union[str, bytes]) -> bytes:
    if isinstance(s, (bytes, bytearray)):
        return s
    return s.encode('utf-8')


def to_str(b: Union[bytes, str]) -> str:
    if isinstance(b, str):
        return b
    return b.decode('utf-8')


def random_key(length: int) -> str:
    if not isinstance(length, int):
        raise Exception(f'nbytes {length} type {type(length)} not int')
    return secrets.token_hex(nbytes=length)[0:length]


def camelize2underline(camelize_words, separator="_"):
    u"""
    驼峰转下划线的方法；
    :param camelize_words: 需要转化的字符串
    :param separator: _
    :return: 字符串
    """
    sep = separator + r'\1'
    if not camelize_words:
        return ""
    if not isinstance(camelize_words):
        raise ValueError('camelize_words {} must be str'.format(camelize_words))
    pattern = re.compile(r'([A-Z]{1})')
    sub = re.sub(pattern, sep, camelize_words).lower()
    return sub


def is_chinese_name(value: str) -> bool:
    u"""
    校验姓名是否是由汉字或汉字和·组成（王大锤 or 买买提·古力娜扎·迪丽热巴）
    :param value: name type is str
    :return:bool  符合格式的name返回 True 否则返回False
    """
    if not isinstance(value, str):
        raise ValueError("value {} type must be string".format(value))
    return PATTERN_CHINESE_NAME.match(value) is not None


def clean_str(s: str, striped='') -> str:
    return ''.join(str(s).strip(striped).split())


if __name__ == '__main__':
    print(unique_id())



import hashlib
try:
    from utils.ustring import to_bytes
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.ustring import to_bytes

def md5(plaintext: str) -> str:
    plaintext = plaintext if isinstance(plaintext, (bytes, bytearray)) else to_bytes(plaintext)
    md5 = hashlib.md5()
    md5.update(plaintext)
    s = md5.hexdigest()
    return s


if __name__ == '__main__':
    print(md5('123445'))



from typing import Dict

import jwt


class Jwt:

    @staticmethod
    def encode(data: Dict) -> str:
        return jwt.encode(data, 'secret', algorithm='HS256')

    @staticmethod
    def decode(encoded: str) -> Dict:
        return jwt.decode(encoded, 'secret', algorithms=['HS256'])



import os
from typing import Dict, Any

try:
    from service.booldata.base import BoolDataBase
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.booldata.base import BoolDataBase

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class Bankcard5Eles(BoolDataBase):
    service = 'bankcard_five_elements_service'
    mode = 'bankcard_five_elements_mode'

    @classmethod
    def gen_biz_data(cls, name: str, ident_number: str, bankcard: str, phone: str, account_type='1') -> Dict[str, Any]:
        return {
            "name": name,
            "ident_number": ident_number,
            "bankcard": bankcard,
            "account_type": account_type,
            "phone": phone,
            "service": cls.service,
            "mode": cls.mode
        }

    def set_biz_data(self, name: str, ident_number: str, bankcard: str, phone: str, account_type='1') -> BoolDataBase:
        """
        support, Bankcard5Eles().set_biz_data().request()
        :param name:
        :param ident_number:
        :param bankcard:
        :param phone:
        :param account_type:
        :return:
        """
        self.biz_data = self.gen_biz_data(name, ident_number, bankcard, phone, account_type=account_type)
        return self


if __name__ == '__main__':
    from service.booldata.gitignore import TEST_ACCOUNT

    s = Bankcard5Eles(TEST_ACCOUNT['rsa_prv_key'], TEST_ACCOUNT['aes_key'], TEST_ACCOUNT['company_uuid'],
                      Bankcard5Eles.urls['prod']).set_biz_data('**', '**',
                                                               '**', '**').request()
    list.count()
    print(s)



try:
    from service.tonglian.merchant_fraud_blacklist import MerchantFraudBlacklist
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.tonglian.merchant_fraud_blacklist import MerchantFraudBlacklist


# -*- coding: utf-8 -*-
import base64
import json
from typing import Any
from typing import Tuple
from typing import Union

from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Signature import pkcs1_15

try:
    from utils.ustring import to_str, to_bytes
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.ustring import to_str, to_bytes


class TypeConvertedError(Exception):
    pass


class RSAEncryption(object):
    """
    RSA/SHA256（摘要）
    """

    @staticmethod
    def import_key(rsa_key: str):
        return RSA.import_key(rsa_key) if rsa_key.startswith('---') else RSA.importKey(base64.b64decode(rsa_key))

    @staticmethod
    def data2bytes(data: Any) -> bytes:
        try:
            if isinstance(data, (bytes, bytearray)):
                return data
            if isinstance(data, str):
                return to_bytes(data)
            else:
                return to_bytes(json.dumps(data, sort_keys=True))
        except Exception as e:
            raise TypeConvertedError(e)

    @staticmethod
    def gen_pair_keys(length=2048) -> Tuple[str, str]:
        print(f'private key length is {length}')
        p = RSA.generate(bits=length)
        return to_str(p.publickey().export_key()), to_str(p.export_key())

    @classmethod
    def gen_signature(cls, data: Any, raw_private_key: str, hash_type=SHA256, returned_base64=True) -> Union[
        str, bytes]:
        private_key = cls.import_key(raw_private_key)
        data = cls.data2bytes(data)
        msg_hash = hash_type.new(data)
        signature = pkcs1_15.new(private_key).sign(msg_hash)
        return to_str(base64.b64encode(signature)) if returned_base64 else signature

    @classmethod
    def verify_signature(cls, raw_data: str, signature: str, raw_public_key: str) -> bool:
        try:
            public_key = cls.import_key(raw_public_key)
            raw_data = cls.data2bytes(raw_data)
            h = SHA256.new(raw_data)
            virified = pkcs1_15.new(public_key).verify(h, base64.b64decode(signature))
            if virified is None:
                return True
            else:
                print(virified)
                return False
        except (ValueError, TypeError) as e:
            import traceback
            traceback.print_exc()
            return False


if __name__ == '__main__':
    RSAEncryption.gen_pair_keys()



import base64
import hashlib
import hmac
from typing import Union

try:
    from utils.ustring import to_bytes, to_str
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.ustring import to_bytes, to_str

def hash_hmac(key: Union[str, bytes], msg: Union[str, bytes], digestmod=hashlib.sha1,
              returned_value_type: Union[str, bytes] = str, need_base64_encode=False) -> Union[
    str, bytes]:
    obj = hmac.new(to_bytes(key), to_bytes(msg), digestmod=digestmod)
    if need_base64_encode:
        return to_str(base64.b64encode(obj.digest()))
    if returned_value_type == str:
        return obj.hexdigest()
    elif returned_value_type in (bytes, bytearray):
        return obj.digest()
    else:
        raise NotImplementedError(f'returned_value_type {returned_value_type} is not supported')


def hash_md5(plaintext: str) -> str:
    m = hashlib.md5()
    m.update(to_bytes(plaintext))
    return m.hexdigest()



try:
    from service.suixingfu.bankcard_four_elements import BankcardFourElements
    from service.suixingfu.bankcard_triple_elements import BankcardTripleElements
    from service.suixingfu.merchant_fraud_blacklist import MerchantFraudBlacklist
    from service.suixingfu.network_triple_elements import NetworkTripleElements
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.suixingfu.bankcard_four_elements import BankcardFourElements
    from pysubway.service.suixingfu.bankcard_triple_elements import BankcardTripleElements
    from pysubway.service.suixingfu.merchant_fraud_blacklist import MerchantFraudBlacklist
    from pysubway.service.suixingfu.network_triple_elements import NetworkTripleElements


import hashlib
from collections import OrderedDict
from typing import Dict, Any, Callable, Optional

import requests

try:
    from utils.ustring import to_bytes
    from utils.utime import timestamp
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.ustring import to_bytes
    from pysubway.utils.utime import timestamp


class ShanghaishuheBase:
    url = 'http://auth.shuhe360.cn:8801/auth/auth-core'
    auth_elements = {
        # 银行卡三要素
        'bankcard_triple_eles': "1101",
        # 银行卡四要素
        'bankcard_four_eles': "1111",
        # 运营商三要素
        'network_triple_eles': "1110",
        # 银行卡三要素详版
        'bankcard_triple_eles_detail': "1101#L",
        # 银行卡四要素详版
        'bankcard_four_eles_detail': "1111#L",
        # 身份二要素认证
        'idcard_two_eles': "1100",
    }
    unsigned_data_order = 'accountName|authElement|bankCardNo|idCardNo|idCardType|phoneNo|expired|cvn2|serialNo|userName|accountPassword'.split(
        '|')

    def __init__(self, account_name: str, account_pwd: str, url: str, version: str = '01'):
        self.url = url
        self.account_name = account_name
        self.account_pwd = account_pwd
        self.version = version
        self.send_data = None

    def gen_send_data(self, **kwargs) -> Dict[str, str]:
        raise NotImplementedError()

    def set_send_data(self, **kwargs) -> 'ShanghaishuheBase':
        self.send_data = self.gen_send_data(**kwargs)
        return self

    @classmethod
    def fmt_func_bak(cls, data: Dict[str, Any]):
        result = OrderedDict()
        for i in cls.unsigned_data_order:
            result[i] = str(data[i]) if i in data else ''
        return '|'.join(result.values())

    @staticmethod
    def fmt_func(accountName='', authElement='', bankCardNo='', idCardNo='', idCardType='', phoneNo='', expired='',
                 cvn2='', serialNo='', userName='', accountPassword=''):
        """
        committed by xiazhi, it's great
        :return:
        """
        return "|".join((accountName, authElement, bankCardNo, idCardNo, idCardType, phoneNo, expired,
                         cvn2, serialNo, userName, accountPassword))

    def gen_sign(self, data: Dict[str, str], fmt_func: Callable = None) -> str:
        """
        签名值为：
        SHA1(accountName|authElement|bankCardNo|idCardNo|idCardType|phoneNo|expired|cvn2|serialNo|userName|accountPassword)组合后的字符串通过SHA1摘要（大写），如无数据则
        采用||中间不留空。
        SHA1的计算小工具可参考：https://1024tools.com/hash
        :param data:
        :param fmt_func:
        :return:
        """
        data['accountPassword'] = self.account_pwd
        fmted = fmt_func(data) if fmt_func else self.fmt_func_bak(data)
        print(fmted)
        return hashlib.sha1(to_bytes(fmted)).hexdigest().upper()

    def request(self, send_data: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        send_data = send_data or self.send_data
        print('send_data', send_data)
        response = requests.post(self.url, json=send_data)
        if response.ok:
            return response.json()
        raise SystemError(f'response {response.text}')


class BankcardFourEles(ShanghaishuheBase):

    def gen_send_data(self, idCardNo: str = '', userName: str = '', bankCardNo: str = '', phoneNo: str = '',
                      idCardType='01') -> Dict[str, str]:
        kwargs = {k: v for k, v in locals().items() if k not in ('self', 'cls')}
        print('kwargs', kwargs)
        if '' in kwargs.values():
            raise ValueError(f"invalid args kwargs {kwargs}")
        kwargs.update({
            'authElement': self.auth_elements['bankcard_four_eles'],
            'accountName': self.account_name,
            'serialNo': timestamp(precision='ms'),
            'version': self.version,
        })
        sign = self.gen_sign(kwargs)
        kwargs['sign'] = sign
        self.send_data = kwargs
        return kwargs


if __name__ == '__main__':
    from service.shanghaishuhe.gitignore import SHANGHAI_SHUHE_ACCOUNT_INFO

    s = BankcardFourEles(SHANGHAI_SHUHE_ACCOUNT_INFO['account_name'], SHANGHAI_SHUHE_ACCOUNT_INFO['account_pwd'],
                         BankcardFourEles.url).set_send_data(idCardNo='**', userName='**',
                                                             bankCardNo='**', phoneNo='**',
                                                             idCardType='**').request()
    print(s)



import json
from typing import Dict, Any, Optional

import requests

try:
    from utils import udict
    from utils.encryption.aes import AESCrypt
    from utils.encryption.rsa2 import RSAEncryption
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils import udict
    from pysubway.utils.encryption.aes import AESCrypt
    from pysubway.utils.encryption.rsa2 import RSAEncryption


class BoolDataBase:
    urls = {
        'local': 'http://localhost:8000/api/lightning/product/query',
        'sit': 'http://guard.shouxin168.net/api/lightning/product/query',
        'prod': 'https://guard.shouxin168.com/api/lightning/product/query'
    }

    def __init__(self, rsa_prv_key: str, aes_key: str, company_uuid: str, url: str = ''):
        self.url = url or self.urls['prod']
        print(f'use url {self.url}')
        self.rsa_prv_key = rsa_prv_key
        self.aes_key = aes_key
        self.company_uuid = company_uuid
        self.resp_data = None
        self.biz_data = None

    @classmethod
    def gen_biz_data(cls, *args) -> Dict[str, Any]:
        raise NotImplementedError()

    def encrypt_biz_data(self, biz_data: Dict[str, str]) -> str:
        sorted_str = json.dumps(biz_data, sort_keys=True)
        return AESCrypt(self.aes_key).encrypt(sorted_str)

    def gen_sign(self, biz_data: Dict[str, str]) -> str:
        sorted_str = udict.Dict(biz_data).sort_keys()
        return RSAEncryption.gen_signature(sorted_str, self.rsa_prv_key)

    def request(self, biz_data: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        biz_data = biz_data or self.biz_data
        data = {
            'sign': self.gen_sign(biz_data),
            'biz_data': self.encrypt_biz_data(biz_data),
            'institution_id': self.company_uuid,
        }
        resp = requests.post(self.url, data=data)
        if resp.ok:
            self.resp_data = resp.json()
            return self.resp_data
        else:
            raise ValueError(resp.text)

    @staticmethod
    def is_success_request(resp: Dict[str, Any]) -> bool:
        if isinstance(resp, dict) and resp.get('resp_code') == 'SW0000':
            return True
        else:
            print(resp)
            return False

    def is_succeed(self, resp: Dict[str, Any] = None) -> bool:
        """
        the function is same to is_success_request,
        but support to chain operate
        :param resp:
        :return:
        """
        response = self.resp_data if resp is None else resp
        return self.is_success_request(response)



from .upandas import csv2dataframe
from .upandas import pd


def csv2xlsx(old: str, new: str, clean: bool = False) -> None:
    if not clean:
        if not isinstance(old, str) and not old.endswith('csv'):
            raise Exception(f'old {old} file must be csv')
        if not isinstance(new, str) and not new.endswith('xlsx'):
            raise Exception(f'old {new} file must be xlsx')
        pd.read_csv(old).to_excel('./tmp.xlsx')
    else:
        csv2dataframe(old).to_excel(new)


def trim_csv(file: str) -> None:
    csv2dataframe(file).to_csv(file)



from flask_restful import Resource

try:
    from controller.request import Request
    from errors import IncomingDataValueError
    from utils.package import Package
    from utils.utype import *
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.controller.request import Request
    from pysubway.errors import IncomingDataValueError
    from pysubway.utils.package import Package
    from pysubway.utils.utype import *

__all__ = [
    'ViewBase',
]


class ViewBase(Resource):
    request = Request()
    service = None

    @classmethod
    def set_service(cls, package_path: str, prefix: str) -> Dict[str, Any]:
        return Package(package_path).get_classes(prefix)

    def exec_service(self) -> Dict[str, Any]:
        if not self.service:
            pass
        cls_name, method_name, _ = self.request.service
        exec_method = getattr(self.service.get(cls_name), method_name)
        if not exec_method:
            raise IncomingDataValueError(f'class cls_name {cls_name} do not have method {method_name}')
        return exec_method(self.request.data)

    def get(self):
        return 'hello pysubway ...'



try:
    from service.bangsheng.loan_risk import LoanRisk
    from service.bangsheng.lend_detail import LendDetail
    from service.bangsheng.loan_detail import LoanDetail
    from service.bangsheng.overdue_detail import OverdueDetail
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.bangsheng.loan_risk import LoanRisk
    from pysubway.service.bangsheng.lend_detail import LendDetail
    from pysubway.service.bangsheng.loan_detail import LoanDetail
    from pysubway.service.bangsheng.overdue_detail import OverdueDetail


# -*- coding: utf-8 -*-

from flask_sqlalchemy import SQLAlchemy

# from sqlalchemy import create_engine


db = SQLAlchemy(use_native_unicode='utf8')
# engine = create_engine('sqlalchemy-uri', encoding='utf-8', echo=True)


"""
use __bind_key__ binds other db

from sqlalchemy import func

class Company(db.Model):
    __bind_key__ = 'shouwei'

    id = db.Column(db.Integer, primary_key=True)
    uuid = db.Column(db.String(36), nullable=False, unique=True)
    name = db.Column(db.String(32, u'utf8_bin'), nullable=False)
    encryption_type = db.Column(db.Integer, nullable=False)
"""


from flask import request

try:
    from errors import IncomingDataTypeError
    from errors import IncomingDataValueError
    from utils.utype import *
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.errors import IncomingDataTypeError
    from pysubway.errors import IncomingDataValueError
    from pysubway.utils.utype import *


class Request:
    """
    {
      "service":"cls.method",
      "parameter": "...."
    }
    """
    f_service = 'service'

    def __init__(self):
        self._incoming_data = dict()

    @property
    def data(self) -> Dict[str, Any]:
        try:
            if not self._incoming_data:
                self._incoming_data = request.json
            return self._incoming_data
        except Exception as e:
            raise IncomingDataTypeError('incoming data must be json')

    @property
    def service(self) -> Tuple[str, str, Optional[List[str]]]:
        service_val = self.data.get(self.f_service)
        if not isinstance(service_val, str):
            raise IncomingDataValueError('service of incoming data is invalid')
        splited = service_val.split('.')
        if len(splited) < 2:
            raise IncomingDataValueError('service of incoming data is wrong.')
        cls_name, method_name, *reserved = splited
        return cls_name, method_name, reserved



from flask_restful import abort

try:
    from utils.utype import *
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.utype import *


class Output:
    f_resp_code = 'resp_code'
    f_resp_data = 'resp_data'
    f_resp_msg = 'resp_msg'
    code_success = 'SW0000'
    code_login_failed = 'SW0001'
    code_authentication_failed = 'SW0002'
    code_permission_deny = 'SW0100'
    code_format_error = 'SW0200'
    code_incoming_data_error = 'SW0300'
    code_not_found = 'SW9998'
    code_system_error = 'SW9999'
    code_msg = {
        # SW0000 成功
        code_success: "成功",
        # SW00xx 登录、认证
        code_login_failed: "登录失败",
        code_authentication_failed: '认证失败',
        # SW01xx 鉴权
        code_permission_deny: "权限不足",
        # SW02格式
        code_format_error: "格式有误",
        # SW03入参
        code_incoming_data_error: "参数错误",
        code_system_error: "系统错误",
        code_not_found: "404 not found",
    }

    def __init__(self, resp_code: str, *args: Dict[str, Any]):
        """
        :return: {
              "resp_code":"SW0000",
              "resp_data":{},
              "resp_msg":"xxxx"
            }
        """
        if resp_code not in self.code_msg:
            raise Exception(f'unknown resp_code {resp_code}')
        self._resp_code = resp_code
        self._resp_msg = self.code_msg[self._resp_code]
        if len(args) <= 1:
            self._resp_data = args[0] if len(args) == 1 else {}
        else:
            resp_data = {}
            for i in args:
                if not isinstance(i, dict):
                    raise Exception('')
                else:
                    resp_data.update(i)
            self._resp_data = resp_data
        self.data = {
            self.f_resp_code: self._resp_code,
            self.f_resp_data: self._resp_data,
            self.f_resp_msg: self._resp_msg
        }

    @classmethod
    def update_code_msg(cls, *args):
        for i in args:
            if not isinstance(i, dict):
                raise Exception('args must be dict')
            cls.code_msg.update(i)


class Response:

    def __init__(self):
        self._abort_code = {
            "SW0002": 401,
            "SW0300": 400,
            "SW9999": 500
        }

    def update_abort_code(self, *args: Dict[str, int]) -> None:
        for i in args:
            if not isinstance(i, dict):
                raise Exception('args must be dict')
            self._abort_code.update(i)

    def abort(self, data: Dict[str, Any], new_abort_code: Dict[str, int] = None) -> Dict[str, Any]:
        if new_abort_code:
            self.update_abort_code(new_abort_code)
        http_code = self._abort_code.get(data.get(Output.f_resp_code))
        if http_code:
            abort(http_code)
        return data

    @classmethod
    def _resp(cls, code: str, *args: Dict[str, Any]) -> Dict[str, Any]:
        return Output(code, *args).data

    @classmethod
    def success(cls, *args: Dict[str, Any]) -> Dict[str, Any]:
        return cls._resp(Output.code_success, *args)

    @classmethod
    def login_fail(cls):
        """
        登陆失败
        """
        return cls._resp(Output.code_login_failed)

    @classmethod
    def authentication_failed(cls):
        return cls._resp(Output.code_authentication_failed)

    @classmethod
    def permission_deny(cls):
        return cls._resp(Output.code_permission_deny)

    @classmethod
    def format_error(cls):
        return cls._resp(Output.code_format_error)

    @classmethod
    def incoming_data_error(cls):
        return cls._resp(Output.code_incoming_data_error)

    @classmethod
    def system_error(cls):
        """
        权限不足
        """
        return cls._resp(Output.code_system_error)

    @classmethod
    def not_found(cls):
        return cls._resp(Output.code_not_found)



from typing import Dict

try:
    from service.suixingfu.base import SuixingfuBase
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.suixingfu.base import SuixingfuBase


class BankcardFourElements(SuixingfuBase):

    def gen_send_data(self, name: str, idcard: str, bankcard: str, phone: str, verify_type: str = '04') -> Dict[
        str, str]:
        return {
            'name': name, 'identNo': idcard, 'cardNo': bankcard, 'verifyType': verify_type, 'phone': phone
        }



import base64
from typing import Union

try:
    from utils.ustring import to_bytes, to_str
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.ustring import to_bytes, to_str


class Picture:

    @staticmethod
    def to_base64(s: Union[str, bytes, bytearray], file=False) -> str:
        if file and isinstance(s, str):
            with open(s, mode='rb') as f:
                s = f.read()
        else:
            s = to_bytes(s)
        return to_str(base64.b64encode(s))
